import React, { Component } from "react";
import BeldenLogo from "../assets/images/belden-white.png";
import DefaultPartImg from "../assets/images/part.png";
import { NavLink } from "react-router-dom";
import "../assets/stylesheets/cabledetail.scss";

class Cable {
  constructor(cableData) {
    this.partnum = cableData.partnum;
    this.id = cableData.id;
    this.brand = cableData.brand;
    this.position = cableData.position;
    this.description = cableData.description
      ? cableData.image
      : "No description available.";
    this.image = cableData.image
      ? `http://localhost${cableData.image}`
      : DefaultPartImg;
    this.dynamicProps = {};
    this.horizontal = [];
    this.vertical = [];
    this.orderedKeys = []

    Object.keys(cableData).forEach((key) => {
      if (!["partnum", "id", "brand", "position", "image", "short_description", "description", "application", "category"].includes(key)) {
        this.dynamicProps[key] = cableData[key];
      }
    });

    for (let key in this.dynamicProps) {
      let data = this.dynamicProps[key];
      let first = Object.keys(data)[0];
      if (typeof data[first] === "object") {
        this.horizontal.push(key);
      } else {
        this.vertical.push(key);
      }
    }
    this.orderedKeys = this.orderedKeys.concat(this.vertical);
    this.orderedKeys = this.orderedKeys.concat(this.horizontal);
  }
}

export default class CableDetailComponent extends Component {
  constructor(props) {
    super(props);
    this.state = {
      cableDetails: null,
    };
  }

  componentDidMount() {
    const cablePos = window.location.href.split("/").pop();
    this.props.socket.send(
      `{"type":"cable_details","call":"request","data":{"position":["${cablePos}"]}}`
    );
    this.props.socket.addEventListener("message", this.handleMessage);
  }

  handleMessage = (event) => {
    try {
      console.log("Message from server", event.data);
      const message = JSON.parse(event.data);
      const cableDetails = new Cable(message.data.cables[0]);
      this.setState({ cableDetails });
    } catch (error) {
      console.error("Error parsing message from server:", error);
    }
  };

  componentWillUnmount() {
    this.props.socket.removeEventListener("message", this.handleMessage);
  }

  renderHorizontalTable(title, object) {
    let data;
    try {
      data = JSON.parse(object);
    } catch (error) {
      console.error("Error parsing JSON data:", error);
      return <div>Error loading data...</div>;
    }

    const keys = Object.keys(data);
    const maxRows = Math.max(...keys.map((key) => data[key].length));

    const rows = Array.from({ length: maxRows }).map((_, rowIndex) => {
      return (
        <tr className="row" key={rowIndex}>
          {keys.map((key, index) => (
            <td className="body" key={index}>
              {data[key][rowIndex] || ""}
            </td>
          ))}
        </tr>
      );
    });

    return (
      <div className="cable-tables-div" style={{ width: "100%" }}>
        <span className="cable-tables-div-title">{title}</span>
        <table>
          <thead className="thead">
            <tr className="row">
              {keys.map((key, index) => (
                <th className="head" key={index}>
                  {key}
                </th>
              ))}
            </tr>
          </thead>
          <tbody className="tbody">{rows}</tbody>
        </table>
      </div>
    );
  }

  renderVerticalTable(title, object) {
    let data;
    try {
      data = JSON.parse(object);
    } catch (error) {
      console.error("Error parsing JSON data:", error);
      return <div>Error loading data...</div>;
    }

    return (
      <div className="cable-tables-div" style={{ width: "32%" }}>
        <span className="cable-tables-div-title">{title}</span>
        <table>
          {Object.entries(data).map(([key, value], index) => (
            <tr className="row" key={index}>
              <th className="head">{key}</th>
              <td className="body">{value}</td>
            </tr>
          ))}
        </table>
      </div>
    );
  }

  render() {
    const { cableDetails } = this.state;

    return (
      <div className="cable">
        <div className="cable-image">
          <img src={BeldenLogo} alt="Belden" />
        </div>
        <div className="cable-fieldContainer">
          <div className="cable-actions">
            <NavLink to="/browse" className="cable-actions-back">
              <span>⬅️ Back</span>
            </NavLink>
          </div>
          {cableDetails ? (
            <div className="cable-main">
              <img
                className="cable-main-image"
                src={cableDetails.image}
                alt="Cable"
              />
              <div className="cable-main-label">
                <div className="cable-main-name">{cableDetails.partnum}</div>
                <div className="cable-main-description">
                  {cableDetails.description}
                </div>
                <div className="cable-main-brand">
                  <span
                    style={{ color: "black", backgroundColor: "transparent" }}
                  >
                    Brand:{" "}
                  </span>
                  {cableDetails.brand}
                </div>
                <div className="cable-main-position">
                  <span
                    style={{ color: "black", backgroundColor: "transparent" }}
                  >
                    Position:{" "}
                  </span>
                  {cableDetails.position}
                </div>
              </div>
            </div>
          ) : (
            <div className="cable-main"></div>
          )}
          <div className="cable-actions">
            <div className="cable-actions-button">
              <span>➤ Datasheet</span>
              <span>✏️</span>
            </div>
            <div className="cable-actions-button">
              <span>➤ Show</span>
              <span>💡</span>
            </div>
            <div className="cable-actions-button">
              <span>➤ Dispense</span>
              <span>🤲</span>
            </div>
          </div>

          {cableDetails ? (
            <div className="cable-tables">
              {/* {Object.entries(cableDetails.dynamicProps).map(
                ([key, value], index) => {
                  if (cableDetails.horizontal.includes(key)) {
                    return this.renderHorizontalTable(
                      key,
                      JSON.stringify(value)
                    );
                  } else {
                    return this.renderVerticalTable(key, JSON.stringify(value));
                  }
                }
              )} */}

              {cableDetails.orderedKeys.map((key, index) => {
                if (cableDetails.horizontal.includes(key)) {
                  return this.renderHorizontalTable(
                    key,
                    JSON.stringify(cableDetails.dynamicProps[key])
                  );
                } else {
                  return this.renderVerticalTable(
                    key,
                    JSON.stringify(cableDetails.dynamicProps[key])
                  );
                }
              }
              )}
            </div>
          ) : (
            <div className="cable-tables"></div>
          )}
        </div>
      </div>
    );
  }
}
